@@ -8,7 +8,7 @@ options(prompt="R>  ", continue = "    ")
 
 
 ###################################################
-### code chunk number 2: using_tinytest.Rnw:71-77
+### code chunk number 2: using_tinytest.Rnw:70-76
 ###################################################
 lbs2kg <- function(x){
   if ( x < 0 ){
@@ -19,7 +19,7 @@ lbs2kg <- function(x){
 
 
 ###################################################
-### code chunk number 3: using_tinytest.Rnw:80-83
+### code chunk number 3: using_tinytest.Rnw:79-82
 ###################################################
 library(tinytest)
 expect_equal(lbs2kg(1), 1/2.2046)
@@ -27,32 +27,32 @@ expect_error(lbs2kg(-3))
 
 
 ###################################################
-### code chunk number 4: using_tinytest.Rnw:89-90
+### code chunk number 4: using_tinytest.Rnw:88-89
 ###################################################
 isTRUE( expect_true(2 == 1 + 1) )
 
 
 ###################################################
-### code chunk number 5: using_tinytest.Rnw:111-113
+### code chunk number 5: using_tinytest.Rnw:110-112
 ###################################################
 expect_error(lbs2kg(-3), pattern="nonnegative")
 expect_error(lbs2kg(-3), pattern="foo")
 
 
 ###################################################
-### code chunk number 6: using_tinytest.Rnw:127-128
+### code chunk number 6: using_tinytest.Rnw:126-127
 ###################################################
 expect_false( 1 + 1 == 2 )
 
 
 ###################################################
-### code chunk number 7: using_tinytest.Rnw:150-151
+### code chunk number 7: using_tinytest.Rnw:149-150
 ###################################################
 print(expect_equal(1+1, 3), type="short")
 
 
 ###################################################
-### code chunk number 8: using_tinytest.Rnw:158-159 (eval = FALSE)
+### code chunk number 8: using_tinytest.Rnw:157-158 (eval = FALSE)
 ###################################################
 ## ?print.tinytests
 
@@ -64,46 +64,45 @@ run_test_file("test_addOne.R", verbose=FALSE)
 
 
 ###################################################
-### code chunk number 10: using_tinytest.Rnw:199-201
+### code chunk number 10: using_tinytest.Rnw:198-200
 ###################################################
 test_results <- run_test_file("test_addOne.R", verbose=FALSE)
 print(test_results, passes=TRUE)
 
 
 ###################################################
-### code chunk number 11: using_tinytest.Rnw:204-205 (eval = FALSE)
+### code chunk number 11: using_tinytest.Rnw:203-204 (eval = FALSE)
 ###################################################
 ## options(tt.pr.passes=TRUE)
 
 
 ###################################################
-### code chunk number 12: using_tinytest.Rnw:210-211 (eval = FALSE)
+### code chunk number 12: using_tinytest.Rnw:209-210 (eval = FALSE)
 ###################################################
 ## run_test_dir("/path/to/your/test/directory")
 
 
 ###################################################
-### code chunk number 13: using_tinytest.Rnw:218-221
+### code chunk number 13: using_tinytest.Rnw:217-219
 ###################################################
 out <- run_test_dir(system.file("tinytest", package="tinytest")
        , verbose=FALSE)
 
 
-
 ###################################################
-### code chunk number 14: using_tinytest.Rnw:225-226
+### code chunk number 14: using_tinytest.Rnw:223-224
 ###################################################
 head(as.data.frame(out), 3)
 
 
 ###################################################
-### code chunk number 15: using_tinytest.Rnw:231-232
+### code chunk number 15: using_tinytest.Rnw:230-231
 ###################################################
 summary(out)
 
 
 ###################################################
-### code chunk number 16: using_tinytest.Rnw:242-245 (eval = FALSE)
+### code chunk number 16: using_tinytest.Rnw:241-244 (eval = FALSE)
 ###################################################
 ## if ( expect_equal(1 + 1, 2) ){
 ##     expect_true( 2 > 0)
@@ -111,7 +110,7 @@ summary(out)
 
 
 ###################################################
-### code chunk number 17: using_tinytest.Rnw:255-258
+### code chunk number 17: using_tinytest.Rnw:254-257
 ###################################################
 if ( ignore(expect_equal)(1+1, 2) ){
   expect_true(2>0)
@@ -119,32 +118,49 @@ if ( ignore(expect_equal)(1+1, 2) ){
 
 
 ###################################################
-### code chunk number 18: using_tinytest.Rnw:289-290 (eval = FALSE)
+### code chunk number 18: using_tinytest.Rnw:278-284 (eval = FALSE)
+###################################################
+## run_test_dir("/path/to/my/testdir"
+##            , remove_side_effects = FALSE)
+## test_all("/path/to/my/testdir"
+##            , remove_side_effects = FALSE)
+## # Only in tests/tinytest.R:
+## test_package("PACKAGENAME", remove_side_effects=FALSE)
+
+
+###################################################
+### code chunk number 19: using_tinytest.Rnw:291-292 (eval = FALSE)
+###################################################
+## options(tt.collate="C")
+
+
+###################################################
+### code chunk number 20: using_tinytest.Rnw:317-318 (eval = FALSE)
+###################################################
+## setup_tinytest("/path/to/your/package")
+
+
+###################################################
+### code chunk number 21: using_tinytest.Rnw:331-332 (eval = FALSE)
 ###################################################
 ## test_all("/path/to/your/package")
 
 
 ###################################################
-### code chunk number 19: using_tinytest.Rnw:295-296 (eval = FALSE)
+### code chunk number 22: using_tinytest.Rnw:343-344 (eval = FALSE)
 ###################################################
-## build_install_test("/path/to/your/package")
+##   build_install_test("/path/to/your/package")
 
 
 ###################################################
-### code chunk number 20: using_tinytest.Rnw:307-308 (eval = FALSE)
-###################################################
-##   build_install_test()
-
-
-###################################################
-### code chunk number 21: using_tinytest.Rnw:337-339 (eval = FALSE)
+### code chunk number 23: using_tinytest.Rnw:376-378 (eval = FALSE)
 ###################################################
 ## dat <- read.csv("women.csv")
 ## expect_equal(dat, women)
 
 
 ###################################################
-### code chunk number 22: using_tinytest.Rnw:357-360
+### code chunk number 24: using_tinytest.Rnw:393-396
 ###################################################
 if ( at_home() ){
   expect_equal(2, 1+1)
@@ -152,7 +168,30 @@ if ( at_home() ){
 
 
 ###################################################
-### code chunk number 23: using_tinytest.Rnw:374-377 (eval = FALSE)
+### code chunk number 25: using_tinytest.Rnw:407-408
+###################################################
+pr <- options(prompt="   ")
+
+
+###################################################
+### code chunk number 26: using_tinytest.Rnw:410-416 (eval = FALSE)
+###################################################
+## if (requireNamespace("tinytest", quietly=TRUE)){
+##   # Do a local check on the environment
+##   on_cran <- identical(Sys.getenv("ONCRAN"),"TRUE")
+##   # now run tinytest
+##   tinytest::test_package("PACKAGENAME", at_home = !on_cran)
+## }
+
+
+###################################################
+### code chunk number 27: using_tinytest.Rnw:418-419
+###################################################
+options(pr)
+
+
+###################################################
+### code chunk number 28: using_tinytest.Rnw:440-443 (eval = FALSE)
 ###################################################
 ## library(hehe)
 ## library(tinytest)
@@ -160,21 +199,21 @@ if ( at_home() ){
 
 
 ###################################################
-### code chunk number 24: using_tinytest.Rnw:395-404
+### code chunk number 29: using_tinytest.Rnw:461-470
 ###################################################
 # exported, user-visible function
 inch2cm <- function(x){
-	x*conversion_factor("inch")
+  x*conversion_factor("inch")
 }
 # not exported function, package-internal
 conversion_factor <- function(unit){
-	confac <- c(inch=2.54, pound=1/2.2056)
+  confac <- c(inch=2.54, pound=1/2.2056)
   confac[unit]
 }
 
 
 ###################################################
-### code chunk number 25: using_tinytest.Rnw:447-455
+### code chunk number 30: using_tinytest.Rnw:513-521
 ###################################################
 pound2kg <- function(x){
   stopifnot( is.numeric(x) )
@@ -187,7 +226,7 @@ pound2kg <- function(x){
 
 
 ###################################################
-### code chunk number 26: using_tinytest.Rnw:474-480 (eval = FALSE)
+### code chunk number 31: using_tinytest.Rnw:540-546 (eval = FALSE)
 ###################################################
 ##     expect_equal(pound2kg(1), 1/2.2046  )
 ##     # test for expected warning, store output
